package lawOfDemeter;

import java.util.*;

/**
 * The ArgumentBin aspect is part of a Law of Demeter Checker - a set of
 * classes and aspects that verify a given program does not violate
 * the Law of Demeter.  This aspect stores the arguments to methods for later
 * examination by the Checker aspect.
 *
 * @author David H. Lorenz
 * @author Modifications made by Paul Freeman
 */
public aspect ArgumentBin extends Supplier {

  /**
   * Prints the objects in the bin of the currently executing method.
   * Used for debugging.
   */
  public void printBin(){
    printHashMap(targets);
  }

  /**
   * Advice that keeps track of arguments to methods.  Maintains a stack of
   * method arguments that match the call stack of the executing application.
   */
  Stack argStack = new Stack();
  Object around():  // keep track of arguments
    Any.Execution(){
      // before a method executes, we store the arguments of the method
      argStack.push(targets);
      targets = new HashMap();
      addAll(thisJoinPoint.getArgs(), argument + at(thisJoinPoint));
      // arguments include this/self - store the object currently executing
      add(thisJoinPoint.getThis(), receiver + at(thisJoinPoint));
      // then we proceed with the method execution
      Object obj = proceed();
      // after the method executes, we discard the arguments
      targets = (HashMap)argStack.pop();
      return obj;
    }
}


