package lawOfDemeter;

import java.util.*;

/**
 * The Checker aspect is part of a Law of Demeter Checker - a set of
 * classes and aspects that verify a given program does not violate
 * the Law of Demeter.  This aspect verifies that method calls do not
 * violate the Law of Demeter.
 *
 * @author David H. Lorenz
 * @author Modifications made by Paul Freeman
 */

public aspect Checker extends Supplier {
// note we still need to extend Supplier to get access to the at method.

  before(): // check
    // we check method calls because:
    // 1. we need the this and target object
    // 2. we can check more.
    // http://aspectj.org/doc/dist/faq.html#q:comparecallandexecution
    // the meaning of the signature in the execution() and call()
    // pointcut designators (PCD's) differ:
    // the call type depends upon the type of the reference making the call,
    // while the execution type depends on the enclosing class.
    Any.MethodCall() {
      Object thisObj = thisJoinPoint.getThis();
      Object target = thisJoinPoint.getTarget();

      System.out.println("Checking--" + thisJoinPoint + at(thisJoinPoint));

      // get the global "Bin" objects
      ArgumentBin argBin = ArgumentBin.aspectOf();

      // get the "Bin" objects associated with the currently executing object (this)
      boolean isPart = false;
      try{
      // this needs to be "tried" as there might NOT be an aspect associated with this obj
        ImmediatePartBin partBin = ImmediatePartBin.aspectOf(thisObj);
        isPart = partBin.contains(target);
      }catch(org.aspectj.lang.NoAspectBoundException e){
        //do nothing
      }

      // for debugging
//      argBin.printBin();
//      partBin.printBin();

      // if the target is not a preferred supplier -> Violation
      if (!argBin.contains(target) && !isPart)
        System.out.println(" !! LoD Violation !! ");
  }

}